{ 4I28 low level procedures }

type LongIntRec = record
  LowWord  : word;
  HighWord : word;
end;

type WordRec = record
  LowByte : byte;
  HighByte : byte;
end;

type FilterData = record
  KP : word;
  KI : word;
  KD : word;
  IL : word;
end;

type Trajectory = record
  Acceleration : longint;
  Velocity     : longint;
  Position     : longint;
end;

type Motor = record
  LM629PortLocation : word;                    { where the (control and status) port is }
  OutPortLocation : word;                      { need to know for Hbridge enable and YSel }
  I28PPIBLocation: word;                       { for access to I/O bits }
  I28PPICLocation: word;                       { for access to I/O bits }
  MotorEnableBit : byte;                       { which bit is the Hbridge enable bit for this motor }
  MotorYSel : byte;                            { which bit is the YSel bit for this motor }
  LastPosition : longint;                      { previous position (for relative and multi-axis moves }
  Filter : FilterData;                         { tuning parameters }
  MotionMode : word;                           { saved copy of motion mode }
  SamplingInterval : word;                     { saved copy of sampling interval }
  ClockFrequency : longint;                    { LM629 clock frequency for acceleration and velocity scaling }
  EncoderLines : word;                         { number of lines in motor encoder (resolution is 4X this number) }
end;

const

{ these are the offsets from the card base address to the individual ports }

I28LM629_0         =  $00;                     { first LM629 control port }
I28LM629_1         =  $02;                     { second LM629 control port }
I28LM629_2         =  $04;                     { third LM629 control port }
I28LM629_3         =  $06;                     { fourth LM629 control port }
I28IMaskReg        =  $08;                     { Interrupt mask register (D0..D3) D4 is IRQEN }
I28IStatReg        =  $09;                     { Interrupt Status Register (D0..D3) D4 is IRQ pin status }
I28ICommReg        =  $0A;                     { Common register allows writes to selected channels at once }
I28PPIA            =  $0C;                     { port A - outputs Hbridge disable bits, LED, and Int driver enable }
I28PPIB            =  $0D;                     { port B - user I/O }
I28PPIC            =  $0E;                     { port C - user I/O }
I28PPICont         =  $0F;                     { control port for PPI }

LM629DataRegOffset =  $01;                     { LM629 data register accsessed with A0 high }
IRQEnBit           =  $10;                     { IRQ enable bit in IMaskReg }



{ PPI port data and masks }

{ outputs on PPIA }

Motor0Enable      = $01;
Motor1Enable      = $02;
Motor2Enable      = $04;
Motor3Enable      = $08;
Motor0YSel        = $10;
Motor1YSel        = $20;
Motor2YSel        = $40;
Motor3YSel        = $80;

{ And a couple useful PPI mode bytes }
OutAInBC           = $8B;                      { the normal PPIA mode for 7I28 with 16 inputs... }
OutAChInBCl        = $83;                      { the normal PPIA mode for 7I28 with 12 inputs and 4 outputs }

{ basic LM629 command bytes (with  data sheet nmemonics) }

ResetData          =$84;                       { to look for a valid reset }
ResetDataMask      =$BF;

{ 'write' commands, no data }
ResetCmd           = $00;                      { reset the LM629 }
SttCmd             = $01;                      { start motion }
DfhCmd             = $02;                      { define home }
SipCmd             = $03;                      { set index position }
UdfCmd             = $04;                      { update filter }
Port8Cmd           = $05;                      { select 8 bit output (pwm for LM629) }
Port12Cmd          = $06;                      { select 12 bit output (for LM628) }

{ 'write' commands, all followed by data }
LPESCmd            = $1A;                      { stop on error }
LPEICmd            = $1B;                      { interrupt on error }
MskICmd            = $1C;                      { mask interrupts }
RstICmd            = $1D;                      { reset interrupts }
LFilCmd            = $1E;                      { load filter parameters }
LTrjCmd            = $1F;                      { load trajectory }
SBpACmd            = $20;                      { set breakpoint absolute }
SBpRCmd            = $21;                      { set breakpoint relative }

{ 'read' commands }
RddvCmd            = $07;                      { read desired velocity }
RddpCmd            = $08;                      { read desired position }
RdipCmd            = $09;                      { read index position }
RdrpCmd            = $0A;                      { read real position }
RdrvCmd            = $0B;                      { read real velocity }
RdsigsCmd          = $0C;                      { read signals register }
RdsumCmd           = $0D;                      { read integration }

{ status byte masks (same as low byte of signals register except bit 0) }
MotorOffMask       = $80;
BreakPointMask     = $40;
ExcessivePosMask   = $20;
WraparoundMask     = $10;
IndexPulseMask     = $08;
TrajectoryComMask  = $04;
CommandErrorMask   = $02;
LM629BusyMask      = $01;

{ signals register masks }
HostInterruptMask  = $8000;
AccelLoadedMask    = $4000;
UDFExecutedMask    = $2000;
ForwardDirMask     = $1000;
VelocityModeMask   = $0800;
OnTargetMask       = $0400;
TurnOffOnEPEMask   = $0200;
EightBitModeMask   = $0100;
{ next six bits can use status register masks }
AcqNextIdxMask     = $0001;

{ load trajectory command (LTRJ) control bits }
ForwardDirection   = $1000;
VelocityMode       = $0800;
PositionMode       = $0000;
StopSmoothlyCmd    = $0400;
StopAbruptlyCmd    = $0200;
TurnOffMotorCmd    = $0100;
TurnOnMotorCmd     = $0000;
LoadAccBit         = $0020;
AccRelBit          = $0010;
LoadVelBit         = $0008;
VelRelBit          = $0004;
LoadPosBit         = $0002;
PosRelBit          = $0001;
LoadAllTraj        = LoadAccBit or LoadVelBit or LoadPosBit;

{ load filter command (LFIL) control bits }
LoadKPBit          = $08;
LoadKIBit          = $04;
LoadKDBit          = $02;
LoadILBit          = $01;
LoadAllFil         = LoadKPBit or LoadKIBit or LoadKDBit or LoadILBit;


var
ALongInt : longint;
AWord : word;

procedure DisableInterrupts;
begin
  inline($FA);                                 { CLI }
end;

procedure EnableInterrupts;
begin
  inline($FB);                                 { STI }
end;


{******** 'motor' record initialization (assigns motor to card half) ********}

procedure AssignMotor(var motorx : motor; card : word; whichmotor : word);
begin
  case whichMotor of
    0:
    begin
      motorx.LM629PortLocation := card + I28LM629_0;
      motorx.MotorEnableBit := Motor0Enable;
      motorx.MotorYSel := Motor0YSel;
    end;
    1:
    begin
      motorx.LM629PortLocation := card + I28LM629_1;
      motorx.MotorEnableBit := Motor1Enable;
      motorx.MotorYSel := Motor1YSel;
    end;
    2:
    begin
      motorx.LM629PortLocation := card + I28LM629_2;
      motorx.MotorEnableBit := Motor2Enable;
      motorx.MotorYSel := Motor2YSel;
    end;
    3:
    begin
      motorx.LM629PortLocation := card + I28LM629_3;
      motorx.MotorEnableBit := Motor3Enable;
      motorx.MotorYSel := Motor3YSel;
    end;
  end; {case}
  {things common to all motors:}
  motorx.OutPortLocation := card + I28PPIA;
  motorx.I28PPIBLocation := card + I28PPIB;
  motorx.I28PPICLocation := card + I28PPIC;

end; { AssignMotor }


{******** low level routines for auxillary IO ports ********}

procedure InitPPI( card : word);
begin
  port[card + I28PPICont] := OutAChInBCl;          { set to normal directions }
  port[card + I28PPIA] := 0;
end;

function ReadIOBits(motorx : motor) : word;
var
bport,cport : byte;
begin
  bport := port[motorx.I28PPIBLocation];
  cport := port[motorx.I28PPICLocation];
  ReadIOBits := bport + (cport and $0F)*$100;
end;

procedure SetIOBits(motorx: motor; bits: byte);
begin
  port[motorx.I28PPICLocation] := (bits shl 4);
end;

procedure EnableHbridge(var motorx : motor);
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave or motorx.motorenablebit;
  port[motorx.outportlocation] := portsave;
end;

procedure DisableHbridge(var motorx : motor);
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave and (not motorx.motorenablebit);
  port[motorx.outportlocation] := portsave;
end;

procedure EnableAHbridge(var motorx : motor);       {applies only to 7I25}
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave and (not motorx.motorenablebit);
  port[motorx.outportlocation] := portsave;
end;

procedure DisableAHbridge(var motorx : motor);      {applies only to 7I25}
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave or motorx.motorenablebit;
  port[motorx.outportlocation] := portsave;
end;

procedure SelectY(var motorx : motor);      {applies only to 7I28}
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave or motorx.MotorYSel;
  port[motorx.outportlocation] := portsave;
end;

procedure SelectX(var motorx : motor);      {applies only to 7I28}
var
portsave : byte;
begin
  portsave := port[motorx.outportlocation];
  portsave := portsave and not motorx.MotorYSel;
  port[motorx.outportlocation] := portsave;
end;

procedure EnableInterruptDriver(card : word);
var
portsave : byte;
begin
  portsave := port[card + I28IMaskReg];
  portsave := portsave or IRQEnBit;
  port[card + I28IMaskReg] := portsave;
end;

procedure DisableInterruptDriver(card : word);
var
portsave : byte;
begin
  portsave := port[card + I28IMaskReg];
  portsave := portsave and not IRQEnBit;
  port[card + I28PPIA] := portsave;
end;

{******** Lowest level LM629 routines ********}

procedure FooDelay(t : word);
var
loops : word;
dummy : byte;
begin  {_very_ approximate 1 usec loops}
  for loops := 1 to t do
  begin
    dummy := port[21]; {ISA I/O access is about same on all machines}
  end;
end;

function ReadLM629Status(var motorx : motor) : byte ;
begin
  ReadLM629Status := port[motorx.LM629PortLocation];
  FooDelay(500);
  { This nonsense is because of a NS LM629 chip bug -- If you poll the status }
  {  register too fast, the LM629 crashes. This seems to be a feature on newer }
  {  (DC >89 or so) chips }
end;

function LM629Busy(var motorx : motor) : boolean;
begin
  LM629Busy := (port[motorx.LM629PortLocation] and LM629BusyMask <> 0);
end;

function TrajectoryComplete(var motorx : motor) : boolean;
begin
  TrajectoryComplete := (ReadLM629Status(motorx) and (TrajectoryComMask or CommandErrorMask or ExcessivePosMask)) <> 0;
end;

procedure WriteCommand(motorx: motor; cmd : byte);
begin
  while LM629Busy(motorx) do FooDelay(10) {wait until ready};
  port[motorx.LM629PortLocation] := cmd;
end;

procedure WriteDataWord(var motorx : motor ; data : word);
begin
  while LM629Busy(motorx) do {wait till ready};
  port[motorx.LM629PortLocation + LM629DataRegOffset] := hi(data);
  port[motorx.LM629PortLocation + LM629DataRegOffset] := lo(data);
end;

function ReadDataWord(var motorx : motor) : word;
begin
  while LM629Busy(motorx) do {wait till ready};
  WordRec(AWord).HighByte := port[motorx.LM629PortLocation + LM629DataRegOffset];
  WordRec(AWord).LowByte  := port[motorx.LM629PortLocation + LM629DataRegOffset];
  ReadDataWord := AWord;
end;

procedure WriteLongint(var motorx : motor; data : longint);
begin
  ALongInt := data;
  WriteDataWord(motorx,LongIntRec(ALongInt).HighWord);
  WriteDataWord(motorx,LongIntRec(ALongInt).LowWord);
end;

function ReadLongint(var motorx : motor): longint;
begin
  LongIntRec(ALongInt).HighWord := ReadDataWord(motorx);
  LongIntRec(ALongInt).LowWord  := ReadDataWord(motorx);
  ReadLongint := ALongInt;
end;

{******** Miscellaneous LM629 read commands ******** }

function ReadSignals(var motorx : motor) : word;
begin
  WriteCommand(motorx,RdSigsCmd);
  ReadSignals := ReadDataWord(motorx);
end;

function ReadIndexPosition(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdIPCmd);
  ReadIndexPosition := ReadLongint(motorx);
end;

function ReadDesiredPosition(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdDPCmd);
  ReadDesiredPosition := ReadLongint(motorx);
end;

function ReadRealPosition(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdRPCmd);
  ReadRealPosition := ReadLongint(motorx);
end;

function ReadDesiredVelocity(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdDVCmd);
  ReadDesiredVelocity := ReadLongint(motorx);
end;

function ReadRealVelocity(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdRVCmd);
  ReadRealVelocity := ReadLongint(motorx);
end;

function ReadIntegraterSum(var motorx : motor) : longint;
begin
  WriteCommand(motorx,RdSumCmd);
  ReadIntegraterSum := ReadLongint(motorx);
end;

procedure PrintStatus(motorx : motor);
var signals : word;
begin
  writeln('HSTI ACLD UDFT FDIR VMOD ONTG TOEP 8BIT MTRO BKPT EXPS WRAP INDX TRCM NXID');
  signals := ReadSignals(motorx);
  if (signals and HostInterruptMask) <> 0 then write('++++ ') else write('---- ');
  if signals and AccelLoadedMask <> 0 then write('++++ ') else write('---- ');
  if signals and UDFExecutedMask <> 0 then write('++++ ') else write('---- ');
  if signals and ForwardDirMask <> 0 then write('++++ ') else write('---- ');
  if signals and VelocityModeMask <> 0 then write('++++ ') else write('---- ');
  if signals and OnTargetMask <> 0 then write('++++ ') else write('---- ');
  if signals and TurnOffOnEPEMask <> 0 then write('++++ ') else write('---- ');
  if signals and EightBitModeMask <> 0 then write('++++ ') else write('---- ');
  if signals and MotorOffMask <> 0 then write('++++ ') else write('---- ');
  if signals and BreakPointMask <> 0 then write('++++ ') else write('---- ');
  if signals and ExcessivePosMask <> 0 then write('++++ ') else write('---- ');
  if signals and WraparoundMask <> 0 then write('++++ ') else write('---- ');
  if signals and IndexPulseMask <> 0 then write('++++ ') else write('---- ');
  if signals and TrajectoryComMask <> 0 then write('++++ ') else write('---- ');
  if signals and AcqNextIdxMask <> 0 then write('++++ ') else write('---- ');
  writeln;
  writeln('Index Position= ',ReadIndexPosition(motorx));
  writeln('Desired Position= ',ReadDesiredPosition(motorx));
  writeln('RealPosition= ',ReadRealPosition(motorx));
  writeln('Desired Velocity= ',ReadDesiredVelocity(motorx));
  writeln('RealVelocity= ',ReadRealVelocity(motorx));
  writeln('Intergrater sum= ',ReadIntegraterSum(motorx));
end;

{******** Low level LM629 Verbs ********}

function ResetLM629(card : word; whichmotor : word) : boolean ;
var
theport : word;
foobar : word;
begin
  case whichmotor of
    0 : theport :=I28LM629_0 + card;
    1 : theport :=I28LM629_1 + card;
    2 : theport :=I28LM629_2 + card;
    3 : theport :=I28LM629_3 + card;
  end;{case}
  port[theport] := ResetCMd;
  FooDelay(20000)  {wait};                                    { wait for a goodly length of time }
  if (port[theport] and ResetDataMask) <> ResetData then ResetLM629 := false
  else ResetLM629 := true;
end;

procedure TurnOnMotor(var motorx : motor);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,TurnOnMotorCmd);
  WriteCommand(motorx,SttCmd);
end;

{ these are emergency measures... }

procedure TurnOffMotor(var motorx : motor);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,TurnOffMotorCmd);
  WriteCommand(motorx,SttCmd);
end;

procedure StopSmoothly(var motorx : motor);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,StopSmoothlyCmd);
  WriteCommand(motorx,SttCmd);
end;

procedure StopAbruptly(var motorx : motor);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,StopAbruptlyCmd);
  WriteCommand(motorx,SttCmd);
end;

{ these reset with the home and index positions to the current position }

procedure DefineHome(var motorx : motor);
begin
  WriteCommand(motorx,DfhCmd);
  motorx.LastPosition := 0;
end;

procedure SetIndexPosition(motorx : motor);
begin
  WriteCommand(motorx,SipCmd);
end;

{ this command starts a pre-loaded trajectory }

procedure StartTrajectory(var motorx : motor);
begin
  WriteCommand(motorx,SttCmd);
end;

{ this command updates the filter coefficients from their (pre-loaded) buffers }

procedure UpdateFilter(var motorx : motor);
begin
  WriteCommand(motorx,UdfCmd);
end;

procedure Port8Command(var motorx : motor);
begin
  WriteCommand(motorx,Port8Cmd);
end;

procedure LoadPosErrForInt(var motorx : motor; maxerr : word);
begin
  WriteCommand(motorx,LPEICmd);
  WriteDataWord(motorx,maxerr);
end;

procedure LoadPosErrForStop(var motorx : motor; maxerr : word);
begin
  WriteCommand(motorx,LPESCmd);
  WriteDataWord(motorx,maxerr);
end;

procedure SetBreakpointAbsolute(var motorx : motor; position : longint);
begin
  WriteCommand(motorx,SBpACmd);
  WriteLongint(motorx,position);
end;

procedure SetBreakpointRelative(var motorx : motor; position : longint);
begin
  WriteCommand(motorx,SBpRCmd);
  WriteLongint(motorx,position);
end;

procedure MaskInterrupts(var motorx : motor; mask : byte);
var
worddata : word;
begin
  worddata := mask;
  WriteCommand(motorx,MskICmd);
  WriteDataWord(motorx,worddata);
end;

procedure ResetInterrupts(var motorx : motor; mask : byte);
var
worddata : word;
begin
  worddata := mask;
  WriteCommand(motorx,RstICmd);
  WriteDataWord(motorx,worddata);
end;

{********  procedures for loading filter and trajectory data to LM629 ********}

procedure LoadFilterParameters(var motorx : motor);
var
filtcontword : word;
begin
  WriteCommand(motorx,LFilCmd);
  filtcontword := motorx.SamplingInterval;
  filtcontword := filtcontword or LoadAllFil;
  WriteDataWord(motorx,filtcontword);
  with motorx.Filter do
  begin
    WriteDataWord(motorx,KP);
    WriteDataWord(motorx,KI);
    WriteDataWord(motorx,KD);
    WriteDataWord(motorx,IL);
  end;{with}
end;

procedure LoadKP(var motorx : motor; kp : word);
begin
  WriteCommand(motorx,LFilCmd);
  WriteDataWord(motorx,motorx.samplinginterval or LoadKPBit);
  WriteDataWord(motorx,kp);
end;

procedure LoadKI(var motorx : motor; ki : word);
begin
  WriteCommand(motorx,LFilCmd);
  WriteDataWord(motorx,motorx.samplinginterval or LoadKIBit);
  WriteDataWord(motorx,ki);
end;

procedure LoadKD(var motorx : motor; kd : word);
begin
  WriteCommand(motorx,LFilCmd);
  WriteDataWord(motorx,motorx.samplinginterval or LoadKDBit);
  WriteDataWord(motorx,kd);
end;

procedure LoadIL(var motorx : motor; il : word);
begin
  WriteCommand(motorx,LFilCmd);
  WriteDataWord(motorx,motorx.samplinginterval or LoadILBit);
  WriteDataWord(motorx,il);
end;

procedure LoadTrajectory(var motorx : motor; var traj : trajectory);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,motorx.MotionMode or LoadAllTraj);
  WriteLongint(motorx,traj.acceleration);
  WriteLongint(motorx,traj.velocity);
  WriteLongint(motorx,traj.position);
end;

procedure LoadAcceleration(var motorx : motor; acceleration : longint);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,motorx.MotionMode or LoadAccBit);
  WriteLongint(motorx,acceleration);
end;

procedure LoadVelocity(var motorx : motor; velocity : longint);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,motorx.MotionMode or LoadVelBit);
  WriteLongint(motorx,velocity);
end;

procedure LoadPosition(var motorx : motor; position : longint);
begin
  WriteCommand(motorx,LTrjCmd);
  WriteDataWord(motorx,motorx.motionmode or LoadPosBit);
  WriteLongint(motorx,position);
end;

{******** mid level commands ********}

function Reset4I28(card : word): boolean;
var
resetok : boolean;
begin
  InitPPI(card);                               { set up 8255 port }
  resetok := ResetLM629(card,0);               { reset first LM629 }
  resetok := ResetLM629(card,1) and resetok;   { reset second LM629 }
  resetok := ResetLM629(card,2) and resetok;   { reset third LM629 }
  resetok := ResetLM629(card,3) and resetok;   { reset fourth LM629 }
  Reset4I28 := resetok;
end;

function ResetSingleChannel4I28(card : word): boolean;
var
resetok : boolean;
begin
  InitPPI(card);                               { set up 8255 port }
  ResetSingleChannel4I28 := ResetLM629(card,0);               { reset first LM629 }
end;

procedure SetSamplingInterval(var motorx : motor; sinterval : word);
begin
  sinterval := swap(sinterval);
  sinterval := sinterval and $FF00;
  motorx.SamplingInterval := sinterval;
end;

procedure SetMotionMode(var motorx : motor; mmode : word);
begin
  motorx.MotionMode := mmode;
end;

procedure SetEncoderlines(var motorx : motor; lines : word);
begin
  motorx.EncoderLines := lines;
end;

procedure SetClockFrequency(var motorx : motor; frequency : longint);
begin
  motorx.ClockFrequency := frequency;
end;

procedure LoadVelocityInRPM(var motorx : motor; velocity : real);
var
samplePeriod : real;
countsPerSample : longint;
begin
  samplePeriod := (motorx.SamplingInterval +1)/(motorx.ClockFrequency / 2048);
  countsPerSample := round((samplePeriod * motorx.EncoderLines * (4/60) * velocity * 65536));
  { X 4 because of four transitions per line on encoder, /60 for rpM, X 65536 to make 16 bit number with 16 bit fraction }
  LoadVelocity(motorx,countsPerSample);
end;

function PolledMove(var motorx : motor; traj : trajectory): byte;
begin
  TurnOffMotor(motorx);                        { this nonsense is due to a bug in pre-Jan. 1990 }
  LoadTrajectory(motorx,traj);
  StartTrajectory(motorx);
  FooDelay(300);
  while not TrajectoryComplete(motorx) do;
  motorx.LastPosition := traj.Position;
  PolledMove := ReadLM629Status(motorx);
end;

function PolledXMove(var motorx : motor; newx, velocity, acceleration : longint): byte;
var traj : trajectory;
begin
  Traj.Position := newx;
  Traj.Velocity := velocity;
  Traj.Acceleration := acceleration;
  TurnOffMotor(motorx);                        { this nonsense is due to a bug in pre-Jan. 1990 }
  LoadTrajectory(motorx,traj);
  StartTrajectory(motorx);
  FooDelay(300);
  while not TrajectoryComplete(motorx) do;
  motorx.LastPosition := traj.Position;
  PolledXMove := ReadLM629Status(motorx);
end;

{ this is a non-synchronized 2 axis move (for indexing) - the path is not a straight line }
{ this procedure expects that the velocity and acceleration values have already been loaded }
{ for both motors }
function PolledXYIndex(var motorx : motor; var motory : motor; newx : longint; newy : longint): byte;
var
status : byte;
begin
  LoadPosition(motorx,newx);
  LoadPosition(motory,newy);
  StartTrajectory(motorx);
  StartTrajectory(motory);
  FooDelay(300);
  while not (TrajectoryComplete(motorx) and TrajectoryComplete(motory)) do { wait };
  motorx.LastPosition := newx;
  motory.LastPosition := newy;
  PolledXYIndex := ReadLM629Status(motorx) or ReadLM629Status(motory);
end;

{ this is a synchronized 2 axis move (for profiling) - the path is a strait line }

function PolledXYMove(var motorx : motor; var motory : motor;
newx : longint; newy : longint; velocity : real; acceleration : real): byte;
var
trajx : trajectory;
trajy : trajectory;
delx : real;
dely : real;
distance : real;
scalex : real;
scaley : real;
status : byte;
begin
  delx := newx - motorx.LastPosition;
  dely := newy - motory.LastPosition;
  distance := Sqrt(Sqr(delx)+Sqr(dely));
  scalex := abs(delx/distance);
  scaley := abs(dely/distance);
  trajx.Position := newx;
  trajy.Position := newy;
  trajx.Velocity := round(velocity * scalex);
  trajy.Velocity := round(velocity * scaley);
  trajx.Acceleration := round(acceleration * scalex);
  trajy.Acceleration := round(acceleration * scaley);
  TurnOffMotor(motorx);                        { this nonsense is due to a bug in pre-Jan. 1990 }
  TurnOffMotor(motory);                        { LM62X controllers... you must     }
  ResetInterrupts(motorx,0);                   { make sure there are no pending interrupts}
  ResetInterrupts(motory,0);
  LoadTrajectory(motorx,trajx);                { turn off motor before loading acceleration! }
  LoadTrajectory(motory,trajy);
  DisableInterrupts;
  StartTrajectory(motorx);
  StartTrajectory(motory);
  EnableInterrupts;
  FooDelay(300);
  while not (TrajectoryComplete(motorx) and TrajectoryComplete(motory)) do
  begin
{    PrintStatus(MotorX);
    PrintStatus(MotorY); }
  end; { wait };
  motorx.LastPosition := newx;
  motory.LastPosition := newy;
  PolledXYMove := (ReadLM629Status(motorx) or ReadLM629Status(motory))
end;
